#include <u.h>
#include <libc.h>
#include <draw.h>
#include <thread.h>
#include <cursor.h>
#include <mouse.h>
#include <keyboard.h>
#include <frame.h>
#include <fcall.h>
#include <plumb.h>
#include <bio.h>
#include <regexp.h>
#include "dat.h"
#include "fns.h"

enum
{
	Maxcfg = 64 * 1024
};

typedef struct Ftag Ftag;

struct Ftag{
	Ftag*	next;
	Reprog *	reprog;
	Rune*	text;
	int	ntext;
};

static Ftag *tags;

Rune*
tagforfile(Rune *fname, int nfname, int *lenp)
{
	Ftag *tl;
	Resub match[2];
	Rune *text;

	text = emalloc(sizeof(Rune)*(nfname+1));
	memmove(text, fname, sizeof(Rune)*nfname);
	text[nfname] = 0;
	if(fname != nil)
		for(tl = tags; tl != nil; tl = tl->next){
			memset(match, 0, sizeof(match));
			if(rregexec(tl->reprog, text, match, 2) == 1){
				free(text);
				*lenp = tl->ntext;
				return tl->text;
			}
		}
	free(text);
	return nil;
}

static int
newtag(char *re, char *txt)
{
	Ftag *tag;

	tag = emalloc(sizeof(Ftag));
	tag->reprog = regcomp(re);
	if(tag->reprog == nil){
		free(tag);
		return -1;
	}
	if(txt[0] == '|')
		txt = smprint("%s ", txt+1);
	else
		txt = smprint(" %s ", txt);
	if(txt == nil)
		sysfatal("no memory");
	tag->text = bytetorune(txt, &tag->ntext);
	free(txt);

	tag->next = tags;
	tags = tag;
	return 0;
}

static int taglinenb;

void
regerror(char* e)
{
	fprint(2, "%s: acmetags:%d: %s\n", argv0, taglinenb, e);
	threadexitsall("tag errors");
}

void
acmetagsinit(void)
{
	char fname[100];
	char *ln;
	char *t;
	Biobuf *bin;
	int i;

	snprint(fname, sizeof(fname), "%s/lib/acmetags", home);
	free(home);
	bin = Bopen(fname, OREAD);
	if(bin == nil)
		return;
	for(i = 1; (ln = Brdstr(bin, '\n', 1)) != nil; i++){
		taglinenb = i;
		if(ln[0] == '#'){
			free(ln);
			continue;
		}
		t = utfrune(ln, '\t');
		if(t == nil)
			t = utfrune(ln, ' ');
		if(t == nil){
			fprint(2, "acmetags:%d: no tag text\n", i);
			free(ln);
			continue;
		}
		*t++ = 0;
		while(*t == ' ' || *t == '\t')
			t++;
		if(*t != '\0')
			if(newtag(ln, t) < 0)
				fprint(2, "acmetags:%d: bad regexp\n", i);
		free(ln);
	}
	Bterm(bin);
}
